/*
 * Copyright 2012 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.client;

import static com.redhat.thermostat.client.Translate._;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import com.redhat.thermostat.client.Connection.ConnectionListener;
import com.redhat.thermostat.client.Connection.ConnectionStatus;
import com.redhat.thermostat.client.ui.ConnectionSelectionDialog;
import com.redhat.thermostat.client.ui.MainWindow;
import com.redhat.thermostat.common.Constants;
import com.redhat.thermostat.common.utils.LoggingUtils;

public class Main {

    private static final Logger logger = LoggingUtils.getLogger(Main.class);

    private ClientArgs arguments;
    private Connection connection;
    private UiFacadeFactory uiFacadeFactory;

    private Main(String[] args) {
        this.arguments = new ClientArgs(args);

        if (arguments.useDummyDataSource()) {
            logger.log(Level.CONFIG, "using dummy data");
            connection = new Connection() {
                @Override
                public void disconnect() {
                    /* no op */
                }

                @Override
                public void connect() {
                    /* no op */
                }
            };
        } else {
            connection = new MongoConnection();
        }
    }

    private void showGui() {
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);

        ConnectionSelectionDialog dialog = new ConnectionSelectionDialog((JFrame) null, connection);
        dialog.pack();
        dialog.setModal(true);
        dialog.setVisible(true);

        if (dialog.isCancelled()) {
            return;
        }

        ConnectionListener connectionListener = new ConnectionListener() {
            @Override
            public void changed(ConnectionStatus newStatus) {
                if (newStatus == ConnectionStatus.FAILED_TO_CONNECT) {
                    JOptionPane.showMessageDialog(
                            null,
                            _("CONNECTION_FAILED_TO_CONNECT_DESCRIPTION"),
                            _("CONNECTION_FAILED_TO_CONNECT_TITLE"),
                            JOptionPane.ERROR_MESSAGE);
                    System.exit(Constants.EXIT_UNABLE_TO_CONNECT_TO_DATABASE);
                }
            }
        };

        connection.addListener(connectionListener);
        connection.connect();
        connection.removeListener(connectionListener);

        uiFacadeFactory = new UiFacadeFactoryImpl((MongoConnection) connection);

        MainWindow gui = new MainWindow(uiFacadeFactory);
        gui.pack();
        gui.setVisible(true);
    }

    public static void main(String[] args) {
        LoggingUtils.setGlobalLogLevel(Level.ALL);

        final Main main = new Main(args);
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                main.showGui();
            }
        });
    }

}
